#include <Wire.h>
#include "Adafruit_TCS34725.h"
#include <Adafruit_NeoPixel.h>
 
#define PIN 6
#define TPIXEL 136 //The total amount of pixel's/led's in your connected strip/stick (Default is 60)

#define kSpokesInStrip 8
#define kPixelsInSpoke 17

 #define kMaxDrops 4
  int dropStripIndexes[kMaxDrops];
  int dropPixelIndexes[kMaxDrops];

int switchPin = 10; // switch is connected to pin 10
int val; // variable for reading the pin status
int val2;
int buttonState; // variable to hold the button state
int lightMode = 0; // how many times the button has been pressed
 
Adafruit_NeoPixel strip = Adafruit_NeoPixel(TPIXEL, PIN, NEO_GRB + NEO_KHZ800);
// our RGB -> eye-recognized gamma color
byte gammatable[256];
Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_50MS, TCS34725_GAIN_4X);
 
void setup() {
Serial.begin(9600); // Set up serial communication at 9600bps
pinMode(switchPin, INPUT_PULLUP); // Set the switch pin as input
pinMode(PIN, OUTPUT);
strip.setBrightness(80); //adjust brightness here
buttonState = digitalRead(switchPin); // read the initial state
strip.begin();
strip.show(); // Initialize all pixels to 'off'
if (tcs.begin()) {
Serial.println("Found sensor");
} else {
Serial.println("No TCS34725 found ... check your connections");
while (1); // halt!
}
// thanks PhilB for this gamma table!
// it helps convert RGB colors to what humans see
for (int i=0; i<256; i++) {
float x = i;
x /= 255;
x = pow(x, 2.5);
x *= 255;
gammatable[i] = x;
//Serial.println(gammatable[i]);
}
for (int i=0; i<3; i++){ //this sequence flashes the first pixel three times as a countdown to the color reading.
strip.setPixelColor (0, strip.Color(188, 188, 188)); //white, but dimmer-- 255 for all three values makes it blinding!
strip.show();
delay(1000);
strip.setPixelColor (0, strip.Color(0, 0, 0));
strip.show();
delay(500);
}
uint16_t clear, red, green, blue;
 
tcs.setInterrupt(false); // turn on LED
 
delay(60); // takes 50ms to read
tcs.getRawData(&red, &green, &blue, &clear);
tcs.setInterrupt(true); // turn off LED
Serial.print("C:\t"); Serial.print(clear);
Serial.print("\tR:\t"); Serial.print(red);
Serial.print("\tG:\t"); Serial.print(green);
Serial.print("\tB:\t"); Serial.print(blue);
// Figure out some basic hex code for visualization
uint32_t sum = red;
sum += green;
sum += blue;
sum = clear;
float r, g, b;
r = red; r /= sum;
g = green; g /= sum;
b = blue; b /= sum;
r *= 256; g *= 256; b *= 256;
Serial.print("\t");
Serial.print((int)r, HEX); Serial.print((int)g, HEX); Serial.print((int)b, HEX);
Serial.println();
 
Serial.print((int)r ); Serial.print(" "); Serial.print((int)g);Serial.print(" "); Serial.println((int)b );
 colorWipe(strip.Color(gammatable[(int)r], gammatable[(int)g], gammatable[(int)b]), 0);

  for (int i=0; i<kMaxDrops; i++)
  {
    dropStripIndexes[i] = random(0, kSpokesInStrip);
    dropPixelIndexes[i] = random(0, kPixelsInSpoke*3/4);
  }
}
 
// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(20);
  }
}

// pixel 0 is at apex
// pixel 16 is at end of first strip
// pixel 17 is beginning of the ccw strip looking from above

void loop(){
  Serial.println("Loop started");
  //rainbow(10);
  //rain();
  //rainbowCycle(10);

    // calculate which pixels in the overall strip are to be lit    
    int onStripIndexes[4];
    for (int i=0; i<kMaxDrops; i++)
    {
      onStripIndexes[i] = dropStripIndexes[i]*kPixelsInSpoke + dropPixelIndexes[i];
    }
    
    // loop through every pixel in the strip to set lit or not
    for (int pixelIndex=0; pixelIndex<kPixelsInSpoke*kSpokesInStrip; pixelIndex++)
    {
      // default to off
      uint32_t color = strip.Color(0, 0, 0);
      // test if should be on and update colour if so
      for (int i=0; i<kMaxDrops; i++)
      {
        if (pixelOn == onStripIndexes[i])
        {
          color = strip.Color(0,0,255);
        }
      }
      // set the colour in the strip
      strip.setPixelColor (pixelIndex, color);
    }
    
    // loop through each drop to advance on it's fall to the ground
    for (int i=0; i<kMaxDrops; i++)
    {
      dropPixelIndexes[i]++;
      // have we dropped off the end? if so, form a new drop somewhere else
      if (dropPixelIndexes[i] >= kPixelsInSpoke)
      {
        dropStripIndexes[i] = random(0,kSpokesInStrip);
        dropPixelIndexes[i] = random(0,kPixelsInSpoke);
      }
    }
    strip.show();
    delay(50);
}

void rain_v1()
{
  //randomly select a spoke
  int spokeIndex = 0;
  spokeIndex = random(0,8);
  int startingPixelIndex = spokeIndex*kPixelsInSpoke;
  int endingPixelIndex = startingPixelIndex+kPixelsInSpoke-1;
  // loop through every frame of animation, where i is the LED that animates as the rain drop falls
  for (int i=startingPixelIndex; i<=endingPixelIndex; i++){
    // loop through every pixel in the strip to set what's displayed per frame
    for (int j=0; j<kPixelsInSpoke*kSpokesInStrip; j++){
      strip.setPixelColor (j, strip.Color(0, 0, 0));
      if (i==j){
        strip.setPixelColor (j, strip.Color(0,0,255));
      }
    }
    strip.show();
    delay(30);
  }
}


// Rain Program
 
void rain() {
// Create an array of 20 raindrops
  const int count = 20;
  int pos[count];
// Set each rain drop at the starting gate.
// Signify by a position of -1
for( int i=0; i < count; i++) {
pos[i]=-1;
}
// Main loop. Keep looping until we've done
// enough "frames."
boolean done=false;
int counter = 0;
while(!done) {
// Start by turning all LEDs off:
for(int i=0; i<strip.numPixels(); i++)
strip.setPixelColor(i, 0);
 
// Loop for each rain drop
for( int i=0; i < count; i++) {
// If the drop is out of the starting gate,
// turn on the LED for it.
if( pos[i] >= 0 ) {
strip.setPixelColor(pos[i], strip.Color(0,0,127));
// Move the drop down one row
pos[i] -= 7;
// If we've fallen off the strip, but us back at the starting gate.
if( pos[i] < 0 )
pos[i]=-1;
}
// If this drop is at the starting gate, randomly
// see if we should start it falling.
if ( pos[i] == -1 && random(40) == 0 && counter < 380 ) {
// Pick one of the 6 starting spots to begin falling
pos[i] = 143-random(6);
}
strip.show();
delay(2);
}
 
}
}
//Rainbow Program
void rainbow(uint8_t wait) {
  uint16_t i, j;
 
  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
strip.setPixelColor(i, Wheel((i+j) & 255));
}
strip.show();
    delay(wait);
  }
}

// Rainbow Cycle Program - Equally distributed
void rainbowCycle(uint8_t wait) {
uint16_t i, j;
 
for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
for(i=0; i< strip.numPixels(); i++) {
strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
}
strip.show();
delay(wait);
}
}
// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
if(WheelPos < 85) {
return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
} else if(WheelPos < 170) {
WheelPos -= 85;
return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
} else {
WheelPos -= 170;
return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
}
}
